-- ChatStamp adds configurable timestamps to World Of Warcraft chat windows.
-- Copyright (C) 2008  Justin Weaver (Visit http://piramod.com for contact)
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

ChatStamp = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceHook-2.1", "FuBarPlugin-2.0")

local tablet = AceLibrary("Tablet-2.0")

local defaults = {
   isTwelveHourClock = false,
   showSeconds = true, 
   color = { r = 0.47,
	     g = 0.47,
	     b = 0.47,
	  },
}

-- FuBar Options
ChatStamp.defaultPosition = "CENTER"
ChatStamp.hasIcon = "Interface\\AddOns\\ChatStamp\\icon"
ChatStamp.defaultMinimapPosition = 285
ChatStamp.tooltipHiddenWhenEmpty = true
ChatStamp.cannotDetachTooltip = true
ChatStamp.hasNoColor = true
ChatStamp.hideWithoutStandby = true
ChatStamp.independentProfile = true

ChatStamp.options = {
   type = "group",
   args = {
      ["isTwelveHourClock"] = {
	 name = "Use 12-Hour Format",
	 type = "toggle",
	 desc = "12-hour format instead of 24-hour format",
	 get = "GetIsTwelveHourClock",
	 set = "SetIsTwelveHourClock",
      },
      ["showSeconds"] = {
	 name = "Show Seconds",
	 type = "toggle",
	 desc = "Show seconds in the timestamp.",
	 get = "GetShowSeconds",
	 set = "SetShowSeconds",
      },
      ["color"] = {
	 name ="Timestamp Color",
	 type ="color",
	 desc ="Select Timestamp color",
	 get = "GetColor",
	 set = "SetColor",
      },
   },
}
ChatStamp.OnMenuRequest = ChatStamp.options

ChatStamp:RegisterChatCommand( { "/chatstamp" }, ChatStamp.options )

ChatStamp:RegisterDB("ChatStampDB")
ChatStamp:RegisterDefaults('profile', defaults)

local _G = getfenv(0)

local stampFormat
local dateFormat

function ChatStamp:GetHexColor()
   return string.format("%02x%02x%02x", self.db.profile.color.r * 255, self.db.profile.color.g * 255, self.db.profile.color.b * 255)
end

function ChatStamp:SetupFormating()
   stampFormat = "|cff".. self:GetHexColor() .."[%s]|r %s"
   if self.db.profile.isTwelveHourClock then
      dateFormat = "%I"
   else
      dateFormat = "%H"
   end
   dateFormat = dateFormat .. ":%M"
   if self.db.profile.showSeconds then
      dateFormat = dateFormat .. ":%S"
   end
   if self.db.profile.isTwelveHourClock then
      dateFormat = dateFormat .. " %p"
   end
end

function ChatStamp:GetIsTwelveHourClock()
   return self.db.profile.isTwelveHourClock
end

function ChatStamp:SetIsTwelveHourClock(v)
   self.db.profile.isTwelveHourClock = v
   self:SetupFormating()
end

function ChatStamp:GetColor()
   return self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b
end

function ChatStamp:SetColor(r, g, b)
   self.db.profile.color.r = r
   self.db.profile.color.g = g
   self.db.profile.color.b = b
   self:SetupFormating()
end

function ChatStamp:GetShowSeconds()
   return self.db.profile.showSeconds
end

function ChatStamp:SetShowSeconds(v)
   self.db.profile.showSeconds = v
   self:SetupFormating()
end

function ChatStamp:OnInitialize()
   self:SetupFormating()
end

function ChatStamp:OnEnable()
   for i=1,NUM_CHAT_WINDOWS do
      self:Hook(_G["ChatFrame"..i], "AddMessage", true)
   end
end

function ChatStamp:AddMessage(frame, text, ...)
   if type(text) == "string" then
      text = stampFormat:format(date(dateFormat), text)
   end
   self.hooks[frame].AddMessage(frame, text, ...)
end

function ChatStamp:OnTooltipUpdate()
   tablet:SetHint("Right-Click for options")
end
